==> payara-embedded-all-5.2020.4-sources.jar: org/glassfish/cdi/transaction/TransactionalInterceptorRequired.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2012-2015 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
// Portions Copyright [2016-2018] [Payara Foundation]

package org.glassfish.cdi.transaction;

import static java.util.logging.Level.FINE;
import static javax.transaction.Status.STATUS_MARKED_ROLLBACK;

import java.util.logging.Logger;

import javax.annotation.Priority;
import javax.interceptor.AroundInvoke;
import javax.interceptor.Interceptor;
import javax.interceptor.InvocationContext;
import javax.transaction.TransactionManager;
import javax.transaction.Transactional;
import javax.transaction.TransactionalException;

import com.sun.enterprise.transaction.TransactionManagerHelper;

/**
 * Transactional annotation Interceptor class for Required transaction type, ie
 * javax.transaction.Transactional.TxType.REQUIRED If called outside a transaction context, a new
 * JTA transaction will begin, the managed bean method execution will then continue inside this
 * transaction context, and the transaction will be committed. If called inside a transaction
 * context, the managed bean method execution will then continue inside this transaction context.
 *
 * @author Paul Parkinson
 */
@Priority(Interceptor.Priority.PLATFORM_BEFORE + 200)
@Interceptor
@Transactional(Transactional.TxType.REQUIRED)
public class TransactionalInterceptorRequired extends TransactionalInterceptorBase {

    private static final long serialVersionUID = 1L;
    private static final Logger _logger = Logger.getLogger(CDI_JTA_LOGGER_SUBSYSTEM_NAME, SHARED_LOGMESSAGE_RESOURCE);

    @AroundInvoke
    public Object transactional(InvocationContext ctx) throws Exception {
        _logger.log(FINE, CDI_JTA_REQUIRED);

        if (isLifeCycleMethod(ctx)) {
            return proceed(ctx);
        }

        setTransactionalTransactionOperationsManger(false);

        try {
            boolean isTransactionStarted = false;

            if (getTransactionManager().getTransaction() == null) {
                _logger.log(FINE, CDI_JTA_MBREQUIRED);

                try {
                    getTransactionManager().begin();
                    TransactionManager transactionManager = getTransactionManager();
                    if (transactionManager instanceof TransactionManagerHelper) {
                        ((TransactionManagerHelper) transactionManager).preInvokeTx(true);
                    }

                } catch (Exception exception) {
                    _logger.log(FINE, CDI_JTA_MBREQUIREDBT, exception);

                    throw new TransactionalException(
                            "Managed bean with Transactional annotation and TxType of REQUIRED " +
                            "encountered exception during begin " +
                            exception,
                            exception);
                }

                isTransactionStarted = true;
            }

            Object proceed = null;

            try {
                proceed = proceed(ctx);
            } finally {
                if (isTransactionStarted) {
                    try {
                        TransactionManager transactionManager = getTransactionManager();
                        if (transactionManager instanceof TransactionManagerHelper) {
                            ((TransactionManagerHelper) transactionManager).postInvokeTx(false, true);
                        }

                        // Exception handling for proceed method call above can set TM/TRX as setRollbackOnly
                        if (getTransactionManager().getTransaction().getStatus() == STATUS_MARKED_ROLLBACK) {
                            getTransactionManager().rollback();
                        } else {
                            getTransactionManager().commit();
                        }
                    } catch (Exception exception) {
                        _logger.log(FINE, CDI_JTA_MBREQUIREDCT, exception);

                        throw new TransactionalException(
                                "Managed bean with Transactional annotation and TxType of REQUIRED " +
                                "encountered exception during commit " +
                                exception,
                                exception);
                    }
                }
            }
            return proceed;
        } finally {
            resetTransactionOperationsManager();
        }
    }
}

==> payara-embedded-all-6.2023.8-sources.jar: org/glassfish/cdi/transaction/TransactionalInterceptorRequired.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2012-2015 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
// Portions Copyright [2016-2021] [Payara Foundation]

package org.glassfish.cdi.transaction;

import static java.util.logging.Level.FINE;
import static jakarta.transaction.Status.STATUS_MARKED_ROLLBACK;

import java.util.logging.Logger;

import jakarta.annotation.Priority;
import jakarta.interceptor.AroundInvoke;
import jakarta.interceptor.Interceptor;
import jakarta.interceptor.InvocationContext;
import jakarta.transaction.TransactionManager;
import jakarta.transaction.Transactional;
import jakarta.transaction.TransactionalException;

import com.sun.enterprise.transaction.TransactionManagerHelper;

/**
 * Transactional annotation Interceptor class for Required transaction type, ie
 * jakarta.transaction.Transactional.TxType.REQUIRED If called outside a transaction context, a new
 * JTA transaction will begin, the managed bean method execution will then continue inside this
 * transaction context, and the transaction will be committed. If called inside a transaction
 * context, the managed bean method execution will then continue inside this transaction context.
 *
 * @author Paul Parkinson
 */
@Priority(Interceptor.Priority.PLATFORM_BEFORE + 200)
@Interceptor
@Transactional(Transactional.TxType.REQUIRED)
public class TransactionalInterceptorRequired extends TransactionalInterceptorBase {

    private static final long serialVersionUID = 1L;
    private static final Logger _logger = Logger.getLogger(CDI_JTA_LOGGER_SUBSYSTEM_NAME, SHARED_LOGMESSAGE_RESOURCE);

    @AroundInvoke
    public Object transactional(InvocationContext ctx) throws Exception {
        _logger.log(FINE, CDI_JTA_REQUIRED);

        if (isLifeCycleMethod(ctx)) {
            return proceed(ctx);
        }

        setTransactionalTransactionOperationsManger(false);

        try {
            boolean isTransactionStarted = false;

            if (getTransactionManager().getTransaction() == null) {
                _logger.log(FINE, CDI_JTA_MBREQUIRED);

                try {
                    getTransactionManager().begin();
                    TransactionManager transactionManager = getTransactionManager();
                    if (transactionManager instanceof TransactionManagerHelper) {
                        ((TransactionManagerHelper) transactionManager).preInvokeTx(true);
                    }

                } catch (Exception exception) {
                    _logger.log(FINE, CDI_JTA_MBREQUIREDBT, exception);

                    throw new TransactionalException(
                            "Managed bean with Transactional annotation and TxType of REQUIRED " +
                            "encountered exception during begin " +
                            exception,
                            exception);
                }

                isTransactionStarted = true;
            }

            Object proceed = null;

            try {
                proceed = proceed(ctx);
            } finally {
                if (isTransactionStarted) {
                    try {
                        TransactionManager transactionManager = getTransactionManager();
                        if (transactionManager instanceof TransactionManagerHelper) {
                            ((TransactionManagerHelper) transactionManager).postInvokeTx(false, true);
                        }

                        // Exception handling for proceed method call above can set TM/TRX as setRollbackOnly
                        if (getTransactionManager().getTransaction().getStatus() == STATUS_MARKED_ROLLBACK) {
                            getTransactionManager().rollback();
                        } else {
                            getTransactionManager().commit();
                        }
                    } catch (Exception exception) {
                        _logger.log(FINE, CDI_JTA_MBREQUIREDCT, exception);

                        throw new TransactionalException(
                                "Managed bean with Transactional annotation and TxType of REQUIRED " +
                                "encountered exception during commit " +
                                exception,
                                exception);
                    }
                }
            }
            return proceed;
        } finally {
            resetTransactionOperationsManager();
        }
    }
}
